# -*- coding:utf-8 -*-
import pytest


def _getTarget():
    from miniconfig import ConfiguratorCore
    return ConfiguratorCore


@pytest.mark.parametrize("current_module, symbol_string, import_symbol", [
    ("foo.bar.boo", "moo", "moo"),
    ("foo.bar.boo", "moo.moo", "moo.moo"),
    ("foo.bar.boo", ".moo", "foo.bar.moo"),
    ("foo.bar.boo", "..moo", "foo.moo"),
    ("foo.bar.boo", ".", "foo.bar.boo"),
    ("foo.bar.boo", "..", "foo.bar"),
])
def test_build_import_symbol_string(current_module, symbol_string, import_symbol):
    class module:
        pass
    config = _getTarget()(module=module())
    config.module.__name__ = current_module
    assert config.build_import_symbol_string(symbol_string) == import_symbol


def test_include__function():
    status = [False]

    def includeme(config):
        status[0] = True

    config = _getTarget()()
    config.include(includeme)
    assert status[0] is True


def test_include__module():
    import imp
    import sys

    status = [False]

    def includeme(config):
        status[0] = True

    module = imp.new_module("miniconfig.foo")
    module.includeme = includeme
    sys.modules["miniconfig.foo"] = module

    config = _getTarget()()
    config.include("miniconfig.foo")
    assert status[0] is True


def test_include__module_with_functioname():
    import imp
    import sys

    status = [False]

    def includeme(config):
        status[0] = True

    module = imp.new_module("miniconfig.boo")
    module.myincludeme = includeme
    sys.modules["miniconfig.boo"] = module

    config = _getTarget()()
    config.include("miniconfig.boo:myincludeme")
    assert status[0] is True



# -*- coding:utf-8 -*-
def _getTarget():
    from miniconfig import ConfiguratorCore
    return ConfiguratorCore


def test_order__no_order_option():
    config = _getTarget()()
    L = []

    def one():
        L.append(1)

    def two():
        L.append(2)

    def three():
        L.append(3)

    config.action(one)
    config.action(two)
    config.action(three)

    config.commit()

    assert L == [1, 2, 3]


def test_order__with_order_option():
    from miniconfig import (
        PHASE1_CONFIG,
        PHASE2_CONFIG
    )

    config = _getTarget()()
    L = []

    def one():
        L.append(1)

    def two():
        L.append(2)

    def three():
        L.append(3)

    config.action(one)
    config.action(two, order=PHASE2_CONFIG)
    config.action(three, order=PHASE1_CONFIG)

    config.commit()

    assert L == [3, 2, 1]



# -*- coding:utf-8 -*-
def _getTarget():
    from miniconfig import ConfiguratorCore
    return ConfiguratorCore


def test_directive__action_is_added():
    def do_action(config):
        pass

    config = _getTarget()()

    assert hasattr(config, "do_action") is False
    config.add_directive("do_action", do_action)
    assert hasattr(config, "do_action") is True


def test_directive__define_by_dottedname():
    import imp
    import sys

    finished = [False]

    def do_action_another_module(config):
        finished[0] = True

    module = imp.new_module("miniconfig.moo")
    module.do_action = do_action_another_module
    sys.modules["miniconfig.moo"] = module

    config = _getTarget()()
    config.add_directive("do_action", "miniconfig.moo:do_action")
    assert hasattr(config, "do_action") is True

    config.do_action()
    assert finished[0] is True


def test_directive__action_with_config():
    m = object()
    finished = [False]

    def with_me(config):
        assert config.settings["m"] == m
        finished[0] = True

    config = _getTarget()(settings={"m": m})
    config.add_directive("with_me", with_me)

    config.with_me()

    assert finished[0] is True


def test_directive__action_with_arguments():
    collection = []
    finished = [False]

    def add_item(config, x):
        collection.append(x)
        finished[0] = True

    config = _getTarget()()
    config.add_directive("add_item", add_item)

    config.add_item(10)
    config.add_item(200)

    assert finished[0] is True
    assert collection == [10, 200]


def test_directive__sub_configurator_also_same_control():
    config = _getTarget()()
    finished = [False]

    def includeme(config2):
        assert config2.hello() == "hello"
        finished[0] = True

    def hello(config):
        return "hello"
    config.add_directive("hello", hello)
    config.include(includeme)

    assert finished[0] is True
    assert config.hello() == "hello"


def test_directive__sub_configurator_also_same_control2():
    class WithHelloControl(object):
        def hello(self, config):
            return "hello"

    config = _getTarget()(control=WithHelloControl())
    finished = [False]

    def includeme(config2):
        assert config2.hello() == "hello"
        finished[0] = True

    config.include(includeme)
    assert finished[0] is True
    assert config.hello() == "hello"



# -*- coding:utf-8 -*-
import logging
logger = logging.getLogger(__name__)
from functools import partial
import pkg_resources
import sys


PHASE1_CONFIG = -20
PHASE2_CONFIG = -10


class ConfigurationError(Exception):
    pass


def import_symbol(symbol):
    return pkg_resources.EntryPoint.parse("x=%s" % symbol).load(False)


def caller_module(level=2):
    module_globals = sys._getframe(level).f_globals
    name = module_globals.get("__name__", "__main__")
    return sys.modules[name]


class Control(object):
    pass


class ConfiguratorCore(object):
    def __init__(self, settings=None, module=None, queue=None, control=None):
        self.settings = settings or {}
        self.module = module or caller_module()
        self.queue = queue or []
        self.control = control or Control()

    def build_import_symbol_string(self, fn_or_string):
        if not fn_or_string.startswith("."):
            return fn_or_string

        nodes = self.module.__name__.split(".")
        if fn_or_string.endswith("."):
            fn_or_string = fn_or_string[1:]

        for i, c in enumerate(fn_or_string):
            if c != ".":
                break
            nodes.pop()
        if fn_or_string == "" or fn_or_string.endswith("."):
            return ".".join(nodes)
        return ".".join(nodes) + "." + fn_or_string[i:]

    def include(self, fn_or_string):
        if callable(fn_or_string):
            includeme = fn_or_string
            module = None
        else:
            symbol_string = self.build_import_symbol_string(fn_or_string)
            if ":" in symbol_string:
                module_string, fn_name = symbol_string.split(":", 1)
            else:
                module_string, fn_name = symbol_string, "includeme"
            module = import_symbol(module_string)
            includeme = getattr(module, fn_name)

        config = self.__class__(self.settings,
                                module=module,
                                queue=self.queue,
                                control=self.control)
        return includeme(config)

    def action(self, callback, order=0):
        self.queue.append((order, callback))

    def commit(self):
        for o, callback in sorted(self.queue, key=lambda xs: xs[0]):
            callback()
        self.queue = []

    def maybe_dotted(self, fn_or_string):
        if callable(fn_or_string):
            return fn_or_string
        symbol_string = self.build_import_symbol_string(fn_or_string)
        return import_symbol(symbol_string)

    def add_directive(self, name, fn_or_string):
        fn = self.maybe_dotted(fn_or_string)
        setattr(self.control, name, fn)

    def __getattr__(self, name):
        attr = getattr(self.control, name)
        if callable(attr):
            return partial(attr, self)
        return attr
